#ifndef __XVAL_H
#define __XVAL_H 1
/* expectation values etc. of EF */

#include "defs.h"
#include "hilbert.h"

#ifndef MAXx
#define MAXx 200l   /* max. number of printed coefficients of the EV */
#endif

extern ULong	x_max;	/* max. number printed coefficients 0..MAXx	*/

/* -------------------------------------------------------------------- */
/* --------       analysis of the eigenvectors              ----------- */
/* -------------------------------------------------------------------- */
/* sort coeff. x_i, biggest x_i => r[MAXx-1]
 * mode=0 maximum values  
 * mode=1 minimum values
 * x_p = output format 1-3=collected, 3=output multiplicity
*/
int x_out(FILE *f, scplx *v0, int mode, int x_p, thbuf *hbuf);

/* ---- print biggest of different coefficients  --------- */
int x_sort(FILE *f,scplx *v0,scplx *v1);

double wh(scplx *v0,thbuf *hbuf[B_NUM]); /* expectation value <H> */
double whq(scplx *v0,cplx rh,thbuf *hbuf[B_NUM]); /* deviation (H*v-h*v)/h */

/* mean value <v1|T*v0>/<v0|v0> (v1=v0) and deviation (T|v0>-<T>v0)^2 
 *   |k> = q sum_R^l exp(i*k*R) T_R |v>   ( Tx_R |x> = |x+R> ) 
 *       = q sum_m^l exp(i*k*m) T^m |v>   - Fourier transformation k=2*Pi/l*k
 *   rk  = <v|k>			  q=1/sqrt(sum_m^l exp(i*k*m)^2)=2/l
 */ 
cplx wop_k(scplx *v0,scplx *v1,int ip);
cplx wop_t(scplx *v0,scplx *v1,int ip);
 
// cplx wop_ud(scplx *v0,scplx *v1); /* v1=T*v0  u<->d */
double wss(scplx *v0,scplx *v1,thbuf *hbuf[B_NUM]); /* SS=1/2*(SpSm+SmSp)+S0^2 ss=s(s+1) */
double wjj(scplx *v0,scplx *v1,thbuf *hbuf[B_NUM]); /* Hubbard SuperSpin Lit.? */

/*  ------------------------------------------------------------
 *  use this function to calculate expectationvalues <v0|op|v1>
 *  of your own operator void op(void *param,int num_sites,int *sites,hbuf)
 *  with hbuf=op(cfg,param)
 *  example_call: wopij(&op_sisj,&para,2,sites,v0,v1);
 *  =>  (void (*)(void *,int,int *,thbuf *)) &op_sisj
 */
double wop( void (*op)(void *,int,int *,thbuf *),
   void *para, int num_sites, int *sites, scplx *v0, scplx *v1,
   thbuf *hbuf[B_NUM]);
/* the same but without non-diag-elements
 * = no interprozess communication = much faster using MPI JS-2011-10-24 */
double w_diag_op( void (*op)(void *,int,int *,thbuf *),
   void *para, int num_sites, int *sites, scplx *v0, scplx *v1,
   thbuf *hbuf[B_NUM]);
      
/*
 *  example operator (nu+nd)(i)*(nu+nd)(j) = op_ninj for wop calculation
 *  decouples operator from vector and symmetry 
 *  and makes the program more flexible for users to create new operators
 *  call_by wop(&op_ninj,NULL,2,iw,v0,v1);
 *   this operator is not symmetric against spin inversion (u <=> d) ???
 */
void op_ninj  (void *para,int ns,int *iw,thbuf *hbuf);
void op_nisnjs(void *para,int ns,int *iw,thbuf *hbuf);

/*  --- occupation operator ---
 *  example operator n(i,s) = op_nis for wop calculation
 *  decouples operator from vector and symmetry
 *  and makes the program more flexible for users to create new operators
 *  call_by wop(&op_nis,(void *)&s1,1,&site,v0,v1);
 *   this operator is not symmetric against spin inversion (u <=> d) ???
 *   so we have wrong results for Sud + nu==nd
 *   para is "1" for n(i,up), "2" for n(i,down) and "3" for nu*nd=ud
 */
void op_nis( void *para,int ns,int *iw,thbuf *hbuf); /* nis */
void op_zi(  void *para,int ns,int *iw,thbuf *hbuf); /* Zi=Siz */
void op_zizj(void *para,int ns,int *iw,thbuf *hbuf); /* ZiZj=SizSjz */
#if NOSZSYM /* Jan18 */
void op_xi(  void *para,int ns,int *iw,thbuf *hbuf); /* Xi=Six */
void op_yi(  void *para,int ns,int *iw,thbuf *hbuf); /* Yi=Siy */
void op_xixj(void *para,int ns,int *iw,thbuf *hbuf); /* XiXj=SixSjx */
void op_yiyj(void *para,int ns,int *iw,thbuf *hbuf); /* YiYj=SiySjy */
#endif /* NOSZSYM */
void op_xyixyj  (void *para,int ns,int *iw,thbuf *hbuf); /* XYiXYj */
void op_sisj    (void *para,int ns,int *iw,thbuf *hbuf); /* SiSj */
void op_cicj    (void *para,int ns,int *iw,thbuf *hbuf); /* see xval.c */
void op_cicjckcl(void *para,int ns,int *iw,thbuf *hbuf); /* see xval.c */

/* 1-Marshall-weight (weight of states breaking the marshall rule) */
int mrule(scplx *v0, int *sl, int num_sl, thbuf *hbuf[B_NUM]);

/* evl. zi()-fkt fuer spin1 in modeldef.h definieren! (allgemeiner) */
/* only for dimer model, spin 1 chains , spin-1 = 2 Spin-1/2 = (4n+2)+(4n+3)
   O(z,pi,i,j)=<v1|Szi*exp(sum(i*Pi*Szk,k=i+1..j))*Szj|v0> |Si|=1*/
int stringorder(FILE *f1,scplx *v0);

/* only for spin 1 chains , spin-1 = 2 Spin-1/2 = (4n+2)+(4n+3)
   O(z,pi,i,j)=<v1|Szi*exp(sum(i*Pi*Szk,k=i+1..j))*Szj|v0> |Si|=1*/
int stringorder2(FILE *f1,scplx *v0);

#endif
